@@ -1,17 +1,16 @@
 // -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-
 
 const Clutter = imports.gi.Clutter;
+const GdkPixbuf = imports.gi.GdkPixbuf
 const Gio = imports.gi.Gio;
 const GLib = imports.gi.GLib;
 const Gtk = imports.gi.Gtk;
-const Pango = imports.gi.Pango;
 const Signals = imports.signals;
 const Lang = imports.lang;
 const Meta = imports.gi.Meta;
 const Shell = imports.gi.Shell;
 const St = imports.gi.St;
 const Mainloop = imports.mainloop;
-const Cairo = imports.cairo;
 
 // Use __ () and N__() for the extension gettext domain, and reuse
 // the shell domain with the default _() and N_()
@@ -33,6 +32,7 @@ const Workspace = imports.ui.workspace;
 const Me = imports.misc.extensionUtils.getCurrentExtension();
 const Utils = Me.imports.utils;
 const WindowPreview = Me.imports.windowPreview;
+const AppIconIndicators = Me.imports.appIconIndicators;
 
 let tracker = Shell.WindowTracker.get_default();
 
@@ -65,7 +65,7 @@ let recentlyClickedAppMonitor = -1;
  *
  * - Pass settings to the constructor and bind settings changes
  * - Apply a css class based on the number of windows of each application (#N);
- * - Draw a dot for each window of the application based on the default "dot" style which is hidden (#N);
+ * - Customized indicators for running applications in place of the default "dot" style which is hidden (#N);
  *   a class of the form "running#N" is applied to the AppWellIcon actor.
  *   like the original .running one.
  * - Add a .focused style to the focused app
@@ -78,16 +78,18 @@ var MyAppIcon = new Lang.Class({
     Extends: AppDisplay.AppIcon,
 
     // settings are required inside.
-    _init: function(settings, app, monitorIndex, iconParams) {
+    _init: function(settings, remoteModel, app, monitorIndex, iconParams) {
         // a prefix is required to avoid conflicting with the parent class variable
         this._dtdSettings = settings;
         this.monitorIndex = monitorIndex;
         this._signalsHandler = new Utils.GlobalSignalsHandler();
-        this._nWindows = 0;
-        this._remoteEntries = [];
+        this.remoteModel = remoteModel;
+        this._indicator = null;
 
         this.parent(app, iconParams);
 
+        this._updateIndicatorStyle();
+
         // Monitor windows-changes instead of app state.
         // Keep using the same Id and function callback (that is extended)
         if (this._stateChangedId > 0) {
@@ -95,43 +97,48 @@ var MyAppIcon = new Lang.Class({
             this._stateChangedId = 0;
         }
 
-        this._stateChangedId = this.app.connect('windows-changed',
+        this._windowsChangedId = this.app.connect('windows-changed',
                                                 Lang.bind(this,
                                                           this.onWindowsChanged));
         this._focusAppChangeId = tracker.connect('notify::focus-app',
                                                  Lang.bind(this,
                                                            this._onFocusAppChanged));
-        this._enteredMonitorId = global.screen.connect('window-entered-monitor',
-                                                       Lang.bind(this,
-                                                                 this.onWindowsChanged));
 
-        this._dots = null;
+        // In Wayland sessions, this signal is needed to track the state of windows dragged
+        // from one monitor to another. As this is triggered quite often (whenever a new winow
+        // of any application opened or moved to a different desktop),
+        // we restrict this signal to  the case when 'isolate-monitors' is true,
+        // and if there are at least 2 monitors.
+        if (this._dtdSettings.get_boolean('isolate-monitors') &&
+            Main.layoutManager.monitors.length > 1) {
+            this._signalsHandler.removeWithLabel('isolate-monitors');
+            this._signalsHandler.addWithLabel('isolate-monitors', [
+                global.screen,
+                'window-entered-monitor',
+                Lang.bind(this, this._onWindowEntered)
+            ]);
+        }
+
         this._progressOverlayArea = null;
         this._progress = 0;
 
         let keys = ['apply-custom-theme',
-                   'custom-theme-running-dots',
-                   'custom-theme-customize-running-dots',
-                   'custom-theme-running-dots-color',
-                   'custom-theme-running-dots-border-color',
-                   'custom-theme-running-dots-border-width'];
+                   'running-indicator-style',
+                    ];
 
         keys.forEach(function(key) {
             this._signalsHandler.add([
                 this._dtdSettings,
                 'changed::' + key,
-                Lang.bind(this, this._toggleDots)
+                Lang.bind(this, this._updateIndicatorStyle)
             ]);
         }, this);
 
-        this._toggleDots();
-
         this._dtdSettings.connect('changed::scroll-action', Lang.bind(this, function() {
             this._optionalScrollCycleWindows();
         }));
         this._optionalScrollCycleWindows();
 
-        this._notificationBadge();
         this._numberOverlay();
 
         this._previewMenuManager = null;
@@ -148,23 +155,39 @@ var MyAppIcon = new Lang.Class({
             this._menu.close(false);
 
         // Disconect global signals
-        // stateChangedId is already handled by parent)
+
+        if (this._windowsChangedId > 0)
+            this.app.disconnect(this._windowsChangedId);
+        this._windowsChangedId = 0;
+
         if (this._focusAppChangeId > 0) {
             tracker.disconnect(this._focusAppChangeId);
             this._focusAppChangeId = 0;
         }
 
-        if (this._enteredMonitorId > 0) {
-            global.screen.disconnect(this._enteredMonitorId);
-            this._enteredMonitorId = 0;
-        }
-
         this._signalsHandler.destroy();
 
         if (this._scrollEventHandler)
             this.actor.disconnect(this._scrollEventHandler);
     },
 
+    // TOOD Rename this function
+    _updateIndicatorStyle: function() {
+
+        if (this._indicator !== null) {
+            this._indicator.destroy();
+            this._indicator = null;
+        }
+        this._indicator = new AppIconIndicators.AppIconIndicator(this, this._dtdSettings);
+        this._indicator.update();
+    },
+
+    _onWindowEntered: function(metaScreen, monitorIndex, metaWin) {
+        let app = Shell.WindowTracker.get_default().get_window_app(metaWin);
+        if (app && app.get_id() == this.app.get_id())
+            this.onWindowsChanged();
+    },
+
     _optionalScrollCycleWindows: function() {
         if (this._scrollEventHandler) {
             this.actor.disconnect(this._scrollEventHandler);
@@ -173,7 +196,6 @@ var MyAppIcon = new Lang.Class({
 
         let isEnabled = this._dtdSettings.get_enum('scroll-action') === scrollAction.CYCLE_WINDOWS;
         if (!isEnabled) return;
-
         this._scrollEventHandler = this.actor.connect('scroll-event', Lang.bind(this,
                                                           this.onScrollEvent));
     },
@@ -238,7 +260,7 @@ var MyAppIcon = new Lang.Class({
         if (this._menu && this._menu.isOpen)
             this._menu.update();
 
-        this._updateRunningStyle();
+        this._indicator.update();
         this.updateIconGeometry();
     },
 
@@ -270,54 +292,10 @@ var MyAppIcon = new Lang.Class({
         });
     },
 
-    _toggleDots: function() {
-        if (this._dtdSettings.get_boolean('custom-theme-running-dots') || this._dtdSettings.get_boolean('apply-custom-theme'))
-            this._showDots();
-        else
-            this._hideDots();
-    },
-
-    _showDots: function() {
-        // I use opacity to hide the default dot because the show/hide function
-        // are used by the parent class.
-        this._dot.opacity = 0;
-
-        // Just update style if dots already exist
-        if (this._dots) {
-            this._updateCounterClass();
-            return;
-        }
-
-        this._dots = new St.DrawingArea({x_expand: true, y_expand: true});
-        this._dots.connect('repaint', Lang.bind(this, function() {
-            this._drawCircles(this._dots, Utils.getPosition(this._dtdSettings));
-        }));
-        this._iconContainer.add_child(this._dots);
-        this._updateCounterClass();
-    },
-
-    _hideDots: function() {
-        this._dot.opacity = 255;
-        if (this._dots)
-            this._dots.destroy()
-        this._dots = null;
-    },
-
     _updateRunningStyle: function() {
-        // When using workspace isolation, we need to hide the dots of apps with
-        // no windows in the current workspace
-        if (this._dtdSettings.get_boolean('isolate-workspaces') ||
-            this._dtdSettings.get_boolean('isolate-monitors')) {
-            if (this.app.state != Shell.AppState.STOPPED
-                && this.getInterestingWindows().length != 0)
-                this._dot.show();
-            else
-                this._dot.hide();
-        }
-        else
-            this.parent();
-        this._onFocusAppChanged();
-        this._updateCounterClass();
+        // The logic originally in this function has been moved to
+        // AppIconIndicatorBase._updateDefaultDot(). However it cannot be removed as
+        // it called by the parent constructor.
     },
 
     popupMenu: function() {
@@ -370,12 +348,7 @@ var MyAppIcon = new Lang.Class({
     },
 
     _onFocusAppChanged: function() {
-        // We need to check the number of windows, as the focus might be
-        // happening on another monitor if using isolation
-        if (tracker.focus_app == this.app && this.getInterestingWindows().length != 0)
-            this.actor.add_style_class_name('focused');
-        else
-            this.actor.remove_style_class_name('focused');
+        this._indicator.update();
     },
 
     activate: function(button) {
@@ -595,259 +568,6 @@ var MyAppIcon = new Lang.Class({
         }
     },
 
-    _updateCounterClass: function() {
-        let maxN = 4;
-        this._nWindows = Math.min(this.getInterestingWindows().length, maxN);
-
-        for (let i = 1; i <= maxN; i++) {
-            let className = 'running' + i;
-            if (i != this._nWindows)
-                this.actor.remove_style_class_name(className);
-            else
-                this.actor.add_style_class_name(className);
-        }
-
-        if (this._dots)
-            this._dots.queue_repaint();
-    },
-
-    _drawCircles: function(area, side) {
-        let borderColor, borderWidth, bodyColor;
-
-        if (!this._dtdSettings.get_boolean('apply-custom-theme')
-            && this._dtdSettings.get_boolean('custom-theme-running-dots')
-            && this._dtdSettings.get_boolean('custom-theme-customize-running-dots')) {
-            borderColor = Clutter.color_from_string(this._dtdSettings.get_string('custom-theme-running-dots-border-color'))[1];
-            borderWidth = this._dtdSettings.get_int('custom-theme-running-dots-border-width');
-            bodyColor =  Clutter.color_from_string(this._dtdSettings.get_string('custom-theme-running-dots-color'))[1];
-        }
-        else {
-            // Re-use the style - background color, and border width and color -
-            // of the default dot
-            let themeNode = this._dot.get_theme_node();
-            borderColor = themeNode.get_border_color(side);
-            borderWidth = themeNode.get_border_width(side);
-            bodyColor = themeNode.get_background_color();
-        }
-
-        let [width, height] = area.get_surface_size();
-        let cr = area.get_context();
-
-        // Draw the required numbers of dots
-        // Define the radius as an arbitrary size, but keep large enough to account
-        // for the drawing of the border.
-        let radius = Math.max(width/22, borderWidth/2);
-        let padding = 0; // distance from the margin
-        let spacing = radius + borderWidth; // separation between the dots
-        let n = this._nWindows;
-
-        cr.setLineWidth(borderWidth);
-        Clutter.cairo_set_source_color(cr, borderColor);
-
-        switch (side) {
-        case St.Side.TOP:
-            cr.translate((width - (2*n)*radius - (n-1)*spacing)/2, padding);
-            for (let i = 0; i < n; i++) {
-                cr.newSubPath();
-                cr.arc((2*i+1)*radius + i*spacing, radius + borderWidth/2, radius, 0, 2*Math.PI);
-            }
-            break;
-
-        case St.Side.BOTTOM:
-            cr.translate((width - (2*n)*radius - (n-1)*spacing)/2, height - padding);
-            for (let i = 0; i < n; i++) {
-                cr.newSubPath();
-                cr.arc((2*i+1)*radius + i*spacing, -radius - borderWidth/2, radius, 0, 2*Math.PI);
-            }
-            break;
-
-        case St.Side.LEFT:
-            cr.translate(padding, (height - (2*n)*radius - (n-1)*spacing)/2);
-            for (let i = 0; i < n; i++) {
-                cr.newSubPath();
-                cr.arc(radius + borderWidth/2, (2*i+1)*radius + i*spacing, radius, 0, 2*Math.PI);
-            }
-            break;
-
-        case St.Side.RIGHT:
-            cr.translate(width - padding , (height - (2*n)*radius - (n-1)*spacing)/2);
-            for (let i = 0; i < n; i++) {
-                cr.newSubPath();
-                cr.arc(-radius - borderWidth/2, (2*i+1)*radius + i*spacing, radius, 0, 2*Math.PI);
-            }
-            break;
-        }
-
-        cr.strokePreserve();
-
-        Clutter.cairo_set_source_color(cr, bodyColor);
-        cr.fill();
-        cr.$dispose();
-    },
-
-    _notificationBadge: function() {
-        this._notificationBadgeLabel = new St.Label();
-        this._notificationBadgeBin = new St.Bin({
-            child: this._notificationBadgeLabel,
-            x_align: St.Align.END, y_align: St.Align.START,
-            x_expand: true, y_expand: true
-        });
-        this._notificationBadgeLabel.add_style_class_name('notification-badge');
-        this._notificationBadgeCount = 0;
-        this._notificationBadgeBin.hide();
-
-        this._iconContainer.add_child(this._notificationBadgeBin);
-        this._iconContainer.connect('allocation-changed', Lang.bind(this, this.updateNotificationBadge));
-    },
-
-    updateNotificationBadge: function() {
-        let scaleFactor = St.ThemeContext.get_for_stage(global.stage).scale_factor;
-        let [minWidth, natWidth] = this._iconContainer.get_preferred_width(-1);
-        let logicalNatWidth = natWidth / scaleFactor;
-        let font_size = Math.max(10, Math.round(logicalNatWidth / 5));
-        let margin_left = Math.round(logicalNatWidth / 4);
-
-        this._notificationBadgeLabel.set_style(
-           'font-size: ' + font_size + 'px;' +
-           'margin-left: ' + margin_left + 'px;'
-        );
-
-        this._notificationBadgeBin.width = Math.round(logicalNatWidth - margin_left);
-        this._notificationBadgeLabel.clutter_text.ellipsize = Pango.EllipsizeMode.MIDDLE;
-    },
-
-    _notificationBadgeCountToText: function(count) {
-        if (count <= 9999) {
-            return count.toString();
-        } else if (count < 1e5) {
-            let thousands = count / 1e3;
-            return thousands.toFixed(1).toString() + "k";
-        } else if (count < 1e6) {
-            let thousands = count / 1e3;
-            return thousands.toFixed(0).toString() + "k";
-        } else if (count < 1e8) {
-            let millions = count / 1e6;
-            return millions.toFixed(1).toString() + "M";
-        } else if (count < 1e9) {
-            let millions = count / 1e6;
-            return millions.toFixed(0).toString() + "M";
-        } else {
-            let billions = count / 1e9;
-            return billions.toFixed(1).toString() + "B";
-        }
-    },
-
-    setNotificationBadge: function(count) {
-        this._notificationBadgeCount = count;
-        let text = this._notificationBadgeCountToText(count);
-        this._notificationBadgeLabel.set_text(text);
-    },
-
-    toggleNotificationBadge: function(activate) {
-        if (activate && this._notificationBadgeCount > 0) {
-            this.updateNotificationBadge();
-            this._notificationBadgeBin.show();
-        }
-        else
-            this._notificationBadgeBin.hide();
-    },
-
-    _showProgressOverlay: function() {
-        if (this._progressOverlayArea) {
-            this._updateProgressOverlay();
-            return;
-        }
-
-        this._progressOverlayArea = new St.DrawingArea({x_expand: true, y_expand: true});
-        this._progressOverlayArea.connect('repaint', Lang.bind(this, function() {
-            this._drawProgressOverlay(this._progressOverlayArea);
-        }));
-
-        this._iconContainer.add_child(this._progressOverlayArea);
-        this._updateProgressOverlay();
-    },
-
-    _hideProgressOverlay: function() {
-        if (this._progressOverlayArea)
-            this._progressOverlayArea.destroy();
-        this._progressOverlayArea = null;
-    },
-
-    _updateProgressOverlay: function() {
-        if (this._progressOverlayArea)
-            this._progressOverlayArea.queue_repaint();
-    },
-
-    _drawProgressOverlay: function(area) {
-        let scaleFactor = St.ThemeContext.get_for_stage(global.stage).scale_factor;
-        let [surfaceWidth, surfaceHeight] = area.get_surface_size();
-        let cr = area.get_context();
-
-        let iconSize = this.icon.iconSize * scaleFactor;
-
-        let x = Math.floor((surfaceWidth - iconSize) / 2);
-        let y = Math.floor((surfaceHeight - iconSize) / 2);
-
-        let lineWidth = Math.floor(1.0 * scaleFactor);
-        let padding = Math.floor(iconSize * 0.05);
-        let width = iconSize - 2.0*padding;
-        let height = Math.floor(Math.min(18.0*scaleFactor, 0.20*iconSize));
-        x += padding;
-        y += iconSize - height - padding;
-
-        cr.setLineWidth(lineWidth);
-
-        // Draw the outer stroke
-        let stroke = new Cairo.LinearGradient(0, y, 0, y + height);
-        let fill = null;
-        stroke.addColorStopRGBA(0.5, 0.5, 0.5, 0.5, 0.1);
-        stroke.addColorStopRGBA(0.9, 0.8, 0.8, 0.8, 0.4);
-        Utils.drawRoundedLine(cr, x + lineWidth/2.0, y + lineWidth/2.0, width, height, true, true, stroke, fill);
-
-        // Draw the background
-        x += lineWidth;
-        y += lineWidth;
-        width -= 2.0*lineWidth;
-        height -= 2.0*lineWidth;
-
-        stroke = Cairo.SolidPattern.createRGBA(0.20, 0.20, 0.20, 0.9);
-        fill = new Cairo.LinearGradient(0, y, 0, y + height);
-        fill.addColorStopRGBA(0.4, 0.25, 0.25, 0.25, 1.0);
-        fill.addColorStopRGBA(0.9, 0.35, 0.35, 0.35, 1.0);
-        Utils.drawRoundedLine(cr, x + lineWidth/2.0, y + lineWidth/2.0, width, height, true, true, stroke, fill);
-
-        // Draw the finished bar
-        x += lineWidth;
-        y += lineWidth;
-        width -= 2.0*lineWidth;
-        height -= 2.0*lineWidth;
-
-        let finishedWidth = Math.ceil(this._progress * width);
-        stroke = Cairo.SolidPattern.createRGBA(0.8, 0.8, 0.8, 1.0);
-        fill = Cairo.SolidPattern.createRGBA(0.9, 0.9, 0.9, 1.0);
-
-        if (Clutter.get_default_text_direction() == Clutter.TextDirection.RTL)
-            Utils.drawRoundedLine(cr, x + lineWidth/2.0 + width - finishedWidth, y + lineWidth/2.0, finishedWidth, height, true, true, stroke, fill);
-        else
-            Utils.drawRoundedLine(cr, x + lineWidth/2.0, y + lineWidth/2.0, finishedWidth, height, true, true, stroke, fill);
-
-        cr.$dispose();
-    },
-
-    setProgress: function(progress) {
-        this._progress = Math.min(Math.max(progress, 0.0), 1.0);
-        this._updateProgressOverlay();
-    },
-
-    toggleProgressOverlay: function(activate) {
-        if (activate) {
-            this._showProgressOverlay();
-        }
-        else {
-            this._hideProgressOverlay();
-        }
-    },
-
     _numberOverlay: function() {
         // Add label for a Hot-Key visual aid
         this._numberOverlayLabel = new St.Label();
@@ -999,70 +719,7 @@ var MyAppIcon = new Lang.Class({
     // nautilus desktop window.
     getInterestingWindows: function() {
         return getInterestingWindows(this.app, this._dtdSettings, this.monitorIndex);
-    },
-
-    insertEntryRemote: function(remote) {
-        if (!remote || this._remoteEntries.includes(remote))
-            return;
-
-        this._remoteEntries.push(remote);
-        this._selectEntryRemote(remote);
-    },
-
-    removeEntryRemote: function(remote) {
-        if (!remote || !this._remoteEntries.includes(remote))
-            return;
-
-        this._remoteEntries.splice(this._remoteEntries.indexOf(remote), 1);
-
-        if (this._remoteEntries.length > 0) {
-            this._selectEntryRemote(this._remoteEntries[this._remoteEntries.length-1]);
-        } else {
-            this.setNotificationBadge(0);
-            this.toggleNotificationBadge(false);
-            this.setProgress(0);
-            this.toggleProgressOverlay(false);
-        }
-    },
-
-    _selectEntryRemote: function(remote) {
-        if (!remote)
-            return;
-
-        this._signalsHandler.removeWithLabel('entry-remotes');
-
-        this._signalsHandler.addWithLabel('entry-remotes',
-        [
-            remote,
-            'count-changed',
-            Lang.bind(this, (remote, value) => {
-                this.setNotificationBadge(value);
-            })
-        ], [
-            remote,
-            'count-visible-changed',
-            Lang.bind(this, (remote, value) => {
-                this.toggleNotificationBadge(value);
-            })
-        ], [
-            remote,
-            'progress-changed',
-            Lang.bind(this, (remote, value) => {
-                this.setProgress(value);
-            })
-        ], [
-            remote,
-            'progress-visible-changed',
-            Lang.bind(this, (remote, value) => {
-                this.toggleProgressOverlay(value);
-            })
-        ]);
-
-        this.setNotificationBadge(remote.count());
-        this.toggleNotificationBadge(remote.countVisible());
-        this.setProgress(remote.progress());
-        this.toggleProgressOverlay(remote.progressVisible());
-    },
+    }
 });
 /**
  * Extend AppIconMenu
@@ -1313,81 +970,90 @@ function getInterestingWindows(app, settings, monitorIndex) {
 }
 
 /**
- * Extend ShowAppsIcon
+ * A wrapper class around the ShowAppsIcon class.
  *
  * - Pass settings to the constructor
- * - set label position based on dash orientation
- * - implement a popupMenu based on the AppIcon code
+ * - set label position based on dash orientation (Note, I am reusing most machinery of the appIcon class)
+ * - implement a popupMenu based on the AppIcon code (Note, I am reusing most machinery of the appIcon class)
+ *
+ * I can't subclass the original object because of this: https://bugzilla.gnome.org/show_bug.cgi?id=688973.
+ * thus use this pattern where the real showAppsIcon object is encaptulated, and a reference to it will be properly wired upon
+ * use of this class in place of the original showAppsButton.
  *
- *  I can't subclass the original object because of this: https://bugzilla.gnome.org/show_bug.cgi?id=688973.
- *  thus use this ugly pattern.
  */
-function extendShowAppsIcon(showAppsIcon, settings) {
-    showAppsIcon._dtdSettings = settings;
-    /* the variable equivalent to toggleButton has a different name in the appIcon class
-     (actor): duplicate reference to easily reuse appIcon methods */
-    showAppsIcon.actor =  showAppsIcon.toggleButton;
 
-    // Re-use appIcon methods
-    showAppsIcon._removeMenuTimeout = AppDisplay.AppIcon.prototype._removeMenuTimeout;
-    showAppsIcon._setPopupTimeout = AppDisplay.AppIcon.prototype._setPopupTimeout;
-    showAppsIcon._onButtonPress = AppDisplay.AppIcon.prototype._onButtonPress;
-    showAppsIcon._onKeyboardPopupMenu = AppDisplay.AppIcon.prototype._onKeyboardPopupMenu;
-    showAppsIcon._onLeaveEvent = AppDisplay.AppIcon.prototype._onLeaveEvent;
-    showAppsIcon._onTouchEvent = AppDisplay.AppIcon.prototype._onTouchEvent;
-    showAppsIcon._onMenuPoppedDown = AppDisplay.AppIcon.prototype._onMenuPoppedDown;
+ var ShowAppsIconWrapper = new Lang.Class({
+    Name: 'DashToDock.ShowAppsIconWrapper',
 
+    _init: function(settings) {
+        this._dtdSettings = settings;
+        this.realShowAppsIcon = new Dash.ShowAppsIcon();
 
-    // No action on clicked (showing of the appsview is controlled elsewhere)
-    showAppsIcon._onClicked = function(actor, button) {
-        showAppsIcon._removeMenuTimeout();
-    };
+        /* the variable equivalent to toggleButton has a different name in the appIcon class
+        (actor): duplicate reference to easily reuse appIcon methods */
+        this.actor = this.realShowAppsIcon.toggleButton;
 
-    showAppsIcon.actor.connect('leave-event', Lang.bind(showAppsIcon, showAppsIcon._onLeaveEvent));
-    showAppsIcon.actor.connect('button-press-event', Lang.bind(showAppsIcon, showAppsIcon._onButtonPress));
-    showAppsIcon.actor.connect('touch-event', Lang.bind(showAppsIcon, showAppsIcon._onTouchEvent));
-    showAppsIcon.actor.connect('clicked', Lang.bind(showAppsIcon, showAppsIcon._onClicked));
-    showAppsIcon.actor.connect('popup-menu', Lang.bind(showAppsIcon, showAppsIcon._onKeyboardPopupMenu));
+        // Re-use appIcon methods
+        this._removeMenuTimeout = AppDisplay.AppIcon.prototype._removeMenuTimeout;
+        this._setPopupTimeout = AppDisplay.AppIcon.prototype._setPopupTimeout;
+        this._onButtonPress = AppDisplay.AppIcon.prototype._onButtonPress;
+        this._onKeyboardPopupMenu = AppDisplay.AppIcon.prototype._onKeyboardPopupMenu;
+        this._onLeaveEvent = AppDisplay.AppIcon.prototype._onLeaveEvent;
+        this._onTouchEvent = AppDisplay.AppIcon.prototype._onTouchEvent;
+        this._onMenuPoppedDown = AppDisplay.AppIcon.prototype._onMenuPoppedDown;
 
-    showAppsIcon._menu = null;
-    showAppsIcon._menuManager = new PopupMenu.PopupMenuManager(showAppsIcon);
-    showAppsIcon._menuTimeoutId = 0;
+        // No action on clicked (showing of the appsview is controlled elsewhere)
+        this._onClicked = Lang.bind(this, function(actor, button) {
+            this._removeMenuTimeout();
+        });
 
-    showAppsIcon.showLabel = itemShowLabel;
+        this.actor.connect('leave-event', Lang.bind(this, this._onLeaveEvent));
+        this.actor.connect('button-press-event', Lang.bind(this, this._onButtonPress));
+        this.actor.connect('touch-event', Lang.bind(this, this._onTouchEvent));
+        this.actor.connect('clicked', Lang.bind(this, this._onClicked));
+        this.actor.connect('popup-menu', Lang.bind(this, this._onKeyboardPopupMenu));
 
-    showAppsIcon.popupMenu = function() {
+        this._menu = null;
+        this._menuManager = new PopupMenu.PopupMenuManager(this);
+        this._menuTimeoutId = 0;
+
+        this.showLabel = itemShowLabel;
+    },
+
+    popupMenu: function() {
         /* disable settings on right-click as only Dash to Dock will display extended settings */
         return;
-        showAppsIcon._removeMenuTimeout();
-        showAppsIcon.actor.fake_release();
 
-        if (!showAppsIcon._menu) {
-            showAppsIcon._menu = new MyShowAppsIconMenu(showAppsIcon, showAppsIcon._dtdSettings);
-            showAppsIcon._menu.connect('open-state-changed', Lang.bind(showAppsIcon, function(menu, isPoppedUp) {
+        this._removeMenuTimeout();
+        this.actor.fake_release();
+
+        if (!this._menu) {
+            this._menu = new MyShowAppsIconMenu(this, this._dtdSettings);
+            this._menu.connect('open-state-changed', Lang.bind(this, function(menu, isPoppedUp) {
                 if (!isPoppedUp)
-                    showAppsIcon._onMenuPoppedDown();
+                    this._onMenuPoppedDown();
             }));
-            let id = Main.overview.connect('hiding', Lang.bind(showAppsIcon, function() {
-                showAppsIcon._menu.close();
+            let id = Main.overview.connect('hiding', Lang.bind(this, function() {
+                this._menu.close();
             }));
-            showAppsIcon._menu.actor.connect('destroy', function() {
+            this._menu.actor.connect('destroy', function() {
                 Main.overview.disconnect(id);
             });
-            showAppsIcon._menuManager.addMenu(showAppsIcon._menu);
+            this._menuManager.addMenu(this._menu);
         }
 
-        showAppsIcon.emit('menu-state-changed', true);
+        //this.emit('menu-state-changed', true);
 
-        showAppsIcon.actor.set_hover(true);
-        showAppsIcon._menu.popup();
-        showAppsIcon._menuManager.ignoreRelease();
-        showAppsIcon.emit('sync-tooltip');
+        this.actor.set_hover(true);
+        this._menu.popup();
+        this._menuManager.ignoreRelease();
+        this.emit('sync-tooltip');
 
         return false;
-    };
+    }
+});
+Signals.addSignalMethods(ShowAppsIconWrapper.prototype);
 
-    Signals.addSignalMethods(showAppsIcon);
-}
 
 /**
  * A menu for the showAppsIcon
