@@ -30,7 +30,9 @@ const Utils = Me.imports.utils;
  * DEFAULT:  transparency given by theme
  * FIXED:    constant transparency chosen by user
  * ADAPTIVE: apply 'transparent' style to dock AND panel when
- *           no windows are close to the dock OR panel
+ *           no windows are close to the dock OR panel.
+ *           When dock is hidden, the dock 'transparent' style only
+ *           apply to itself.
  * DYNAMIC:  apply 'transparent' style when no windows are close to the dock
  * */
 const TransparencyMode = {
@@ -195,7 +197,7 @@ var ThemeManager = new Lang.Class({
         else
             this._actor.remove_style_class_name('shrink');
 
-        if (this._settings.get_boolean('custom-theme-running-dots'))
+        if (this._settings.get_enum('running-indicator-style') !== 0)
             this._actor.add_style_class_name('running-dots');
         else
             this._actor.remove_style_class_name('running-dots');
@@ -204,7 +206,7 @@ var ThemeManager = new Lang.Class({
         if (!this._settings.get_boolean('apply-custom-theme')) {
             if (this._settings.get_boolean('force-straight-corner'))
                 this._actor.add_style_class_name('straight-corner');
-            else 
+            else
                 this._actor.remove_style_class_name('straight-corner');
         } else {
             this._actor.remove_style_class_name('straight-corner');
@@ -374,10 +376,14 @@ const Transparency = new Lang.Class({
             global.window_manager,
             'switch-workspace',
             Lang.bind(this, this._updateSolidStyle)
-        ],[
+        ], [
             Main.overview,
             'hiding',
             Lang.bind(this, this._updateSolidStyle)
+        ], [
+            Main.overview,
+            'showing',
+            Lang.bind(this, this._updateSolidStyle)
         ]);
 
         // Window signals
@@ -393,6 +399,8 @@ const Transparency = new Lang.Class({
 
         if (this._actor.get_stage())
             this._updateSolidStyle();
+
+        this.emit('transparency-enabled');
     },
 
     disable: function() {
@@ -407,6 +415,8 @@ const Transparency = new Lang.Class({
                 key.disconnect(id);
             });
         this._trackedWindows.clear();
+
+        this.emit('transparency-disabled');
     },
 
     destroy: function() {
@@ -435,16 +445,23 @@ const Transparency = new Lang.Class({
     },
 
     _updateSolidStyle: function() {
-        if (this._dockIsNear() || this._panelIsNear()) {
+        let isNear = this._dockIsNear() || this._panelIsNear();
+        if (isNear) {
             this._actor.set_style(this._opaque_style);
-            if (this._panel._updateSolidStyle && this._adaptiveEnabled)
-                this._panel._addStyleClassName('solid');
+            if (this._panel._updateSolidStyle && this._adaptiveEnabled) {
+                if (this._settings.get_boolean('dock-fixed') || this._panelIsNear())
+                    this._panel._addStyleClassName('solid');
+                else
+                    this._panel._removeStyleClassName('solid');
+            }
         }
         else {
             this._actor.set_style(this._transparent_style);
             if (this._panel._updateSolidStyle && this._adaptiveEnabled)
                 this._panel._removeStyleClassName('solid');
         }
+
+        this.emit('solid-style-updated', isNear);
     },
 
     _dockIsNear: function() {
@@ -549,6 +566,8 @@ const Transparency = new Lang.Class({
             'border-color: rgba(' +
             this._backgroundColor + ',' + this._opaqueAlphaBorder + ');' +
             'transition-duration: ' + this._opaqueTransition + 'ms;';
+
+        this.emit('styles-updated');
     },
 
     setColor: function(color) {
@@ -650,3 +669,4 @@ const Transparency = new Lang.Class({
             this._panel._onWindowActorAdded(null, win);
     }
 });
+Signals.addSignalMethods(Transparency.prototype);
